@@ -20,7 +20,6 @@
 
 //! Compilation of regexes to VM.
 
-use regex;
 use std::usize;
 
 use crate::analyze::Info;
@@ -162,6 +161,15 @@ impl Compiler {
                 // TODO: might want to have more specialized impls
                 self.compile_delegate(info)?;
             }
+            Expr::NamedBackref(_) => {
+                unreachable!("named backrefs should have been eliminated");
+            }
+            Expr::KeepOut => {
+                self.b.add(Insn::Save(0));
+            }
+            Expr::ContinueFromPreviousMatchEnd => {
+                self.b.add(Insn::ContinueFromPreviousMatchEnd);
+            }
         }
         Ok(())
     }
@@ -203,15 +211,27 @@ impl Compiler {
 
     fn compile_concat(&mut self, info: &Info<'_>, hard: bool) -> Result<()> {
         // First: determine a prefix which is constant size and not hard.
-        let prefix_end = info.children.iter().take_while(|c| c.const_size && !c.hard).count();
+        let prefix_end = info
+            .children
+            .iter()
+            .take_while(|c| c.const_size && !c.hard)
+            .count();
 
         // If incoming difficulty is not hard, the suffix after the last
         // hard child can be done with NFA.
         let suffix_len = if !hard {
-            info.children[prefix_end..].iter().rev().take_while(|c| !c.hard).count()
+            info.children[prefix_end..]
+                .iter()
+                .rev()
+                .take_while(|c| !c.hard)
+                .count()
         } else {
             // Even for hard, we can delegate a const-sized suffix
-            info.children[prefix_end..].iter().rev().take_while(|c| c.const_size && !c.hard).count()
+            info.children[prefix_end..]
+                .iter()
+                .rev()
+                .take_while(|c| c.const_size && !c.hard)
+                .count()
         };
         let suffix_begin = info.children.len() - suffix_len;
 
@@ -522,28 +542,32 @@ mod tests {
 
     use super::*;
     use crate::analyze::analyze;
+    use crate::parse::ExprTree;
     use crate::vm::Insn::*;
     use bit_set::BitSet;
     use matches::assert_matches;
 
     #[test]
     fn jumps_for_alternation() {
-        let expr = Expr::Alt(vec![
-            Expr::Literal {
-                val: "a".into(),
-                casei: false,
-            },
-            Expr::Literal {
-                val: "b".into(),
-                casei: false,
-            },
-            Expr::Literal {
-                val: "c".into(),
-                casei: false,
-            },
-        ]);
-        let backrefs = BitSet::new();
-        let info = analyze(&expr, &backrefs).unwrap();
+        let tree = ExprTree {
+            expr: Expr::Alt(vec![
+                Expr::Literal {
+                    val: "a".into(),
+                    casei: false,
+                },
+                Expr::Literal {
+                    val: "b".into(),
+                    casei: false,
+                },
+                Expr::Literal {
+                    val: "c".into(),
+                    casei: false,
+                },
+            ]),
+            backrefs: BitSet::new(),
+            named_groups: Default::default(),
+        };
+        let info = analyze(&tree).unwrap();
 
         let mut c = Compiler::new(0);
         // Force "hard" so that compiler doesn't just delegate
@@ -617,8 +641,8 @@ mod tests {
     }
 
     fn compile_prog(re: &str) -> Vec<Insn> {
-        let (expr, backrefs) = Expr::parse(re).unwrap();
-        let info = analyze(&expr, &backrefs).unwrap();
+        let tree = Expr::parse_tree(re).unwrap();
+        let info = analyze(&tree).unwrap();
         let prog = compile(&info).unwrap();
         prog.body
     }
