@@ -20,7 +20,7 @@
 
 //! Backtracking VM for implementing fancy regexes.
 //!
-//! Read https://swtch.com/~rsc/regexp/regexp2.html for a good introduction for how this works.
+//! Read <https://swtch.com/~rsc/regexp/regexp2.html> for a good introduction for how this works.
 //!
 //! The VM executes a sequence of instructions (a program) against an input string. It keeps track
 //! of a program counter (PC) and an index into the string (IX). Execution can have one or more
@@ -35,7 +35,7 @@
 //!
 //! A very simple program for the regex `a`:
 //!
-//! ```norun
+//! ```text
 //! 0: Lit("a")
 //! 1: End
 //! ```
@@ -47,7 +47,7 @@
 //!
 //! Example program for the regex `ab|ac`:
 //!
-//! ```norun
+//! ```text
 //! 0: Split(1, 4)
 //! 1: Lit("a")
 //! 2: Lit("b")
@@ -78,13 +78,21 @@ use crate::Error;
 use crate::Result;
 use crate::{codepoint_len, RegexOptions};
 
-const OPTION_TRACE: u32 = 1;
+/// Enable tracing of VM execution. Only for debugging/investigating.
+const OPTION_TRACE: u32 = 1 << 0;
+/// When iterating over all matches within a text (e.g. with `find_iter`), empty matches need to be
+/// handled specially. If we kept matching at the same position, we'd never stop. So what we do
+/// after we've had an empty match, is to advance the position where matching is attempted.
+/// If `\G` is used in the pattern, that means it no longer matches. If we didn't tell the VM about
+/// the fact that we skipped because of an empty match, it would still treat `\G` as matching. So
+/// this option is for communicating that to the VM. Phew.
+pub(crate) const OPTION_SKIPPED_EMPTY_MATCH: u32 = 1 << 1;
 
 // TODO: make configurable
 const MAX_STACK: usize = 1_000_000;
 
 /// Instruction of the VM.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum Insn {
     /// Successful end of program
     End,
@@ -181,10 +189,12 @@ pub enum Insn {
         /// The last group number
         end_group: usize,
     },
+    /// Anchor to match at the position where the previous match ended
+    ContinueFromPreviousMatchEnd,
 }
 
 /// Sequence of instructions for the VM to execute.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Prog {
     /// Instructions of the program
     pub body: Vec<Insn>,
@@ -409,6 +419,7 @@ pub fn run_default(prog: &Prog, s: &str, pos: usize) -> Result<Option<Vec<usize>
 }
 
 /// Run the program with options.
+#[allow(clippy::cognitive_complexity)]
 pub(crate) fn run(
     prog: &Prog,
     s: &str,
@@ -438,6 +449,13 @@ pub(crate) fn run(
                     if option_flags & OPTION_TRACE != 0 {
                         println!("saves: {:?}", state.saves);
                     }
+                    if let Some(&slot1) = state.saves.get(1) {
+                        // With some features like keep out (\K), the match start can be after
+                        // the match end. Cap the start to <= end.
+                        if state.get(0) > slot1 {
+                            state.save(0, slot1);
+                        }
+                    }
                     return Ok(Some(state.saves));
                 }
                 Insn::Any => {
@@ -645,6 +663,11 @@ pub(crate) fn run(
                         }
                     }
                 }
+                Insn::ContinueFromPreviousMatchEnd => {
+                    if ix > pos || option_flags & OPTION_SKIPPED_EMPTY_MATCH != 0 {
+                        break 'fail;
+                    }
+                }
             }
             pc += 1;
         }
@@ -671,7 +694,6 @@ pub(crate) fn run(
 mod tests {
     use super::*;
     use quickcheck::{quickcheck, Arbitrary, Gen};
-    use rand::Rng;
 
     #[test]
     fn state_push_pop() {
@@ -786,11 +808,14 @@ mod tests {
     }
 
     impl Arbitrary for Operation {
-        fn arbitrary<G: Gen>(g: &mut G) -> Self {
-            match g.gen_range(0, 3) {
-                0 => Operation::Push,
-                1 => Operation::Pop,
-                _ => Operation::Save(g.gen_range(0, 5), g.gen_range(0, usize::MAX)),
+        fn arbitrary(g: &mut Gen) -> Self {
+            match g.choose(&[0, 1, 2]) {
+                Some(0) => Operation::Push,
+                Some(1) => Operation::Pop,
+                _ => Operation::Save(
+                    *g.choose(&[0usize, 1, 2, 3, 4]).unwrap(),
+                    usize::arbitrary(g),
+                ),
             }
         }
     }
