@@ -10,9 +10,6 @@
   // Compile failed: InvalidEscape("\\O")
   x2("$\\O", "bb\n", 2, 3);
 
-  // Compile failed: InvalidEscape("\\G")
-  x2("\\G", "", 0, 0);
-
   // Compile failed: InvalidEscape("\\Z")
   x2("\\Z", "", 0, 0);
 
@@ -56,9 +53,6 @@
   // Compile failed: InvalidEscape("\\Z")
   x2("a\\Z", "a", 0, 1);
 
-  // Compile failed: InvalidEscape("\\G")
-  x2("\\Gaz", "az", 0, 2);
-
   // No match found
   x2("(?i:ss)", "\xc3\x9f", 0, 2);
 
@@ -92,18 +86,6 @@
   // No match found
   x2("(?m:.b)", "a\nb", 1, 3);
 
-  // Match found at start 0 and end 1 (expected 0 and 0)
-  x2("|a", "a", 0, 0);
-
-  // Match found at start 0 and end 1 (expected 0 and 0)
-  x2("(|a)", "a", 0, 0);
-
-  // Compile failed: InvalidEscape("\\G")
-  x2("a|\\Gz", "bza", 2, 3);
-
-  // Compile failed: InvalidEscape("\\G")
-  x2("a|\\Gz", "za", 0, 1);
-
   // Compile failed: InvalidEscape("\\Z")
   x2("a|b\\Z", "ba", 1, 2);
 
@@ -320,21 +302,6 @@
   // Compile failed: InvalidEscape("\\O")
   x2("(?-m:\\O)", "\n", 0, 1);
 
-  // Compile failed: InvalidEscape("\\K")
-  x2("\\K", "a", 0, 0);
-
-  // Compile failed: InvalidEscape("\\K")
-  x2("a\\K", "a", 1, 1);
-
-  // Compile failed: InvalidEscape("\\K")
-  x2("a\\Kb", "ab", 1, 2);
-
-  // Compile failed: InvalidEscape("\\K")
-  x2("(a\\Kb|ac\\Kd)", "acd", 2, 3);
-
-  // Compile failed: InvalidEscape("\\K")
-  x2("(a\\Kb|\\Kac\\K)*", "acababacab", 9, 10);
-
   // No match found
   x2("(?:()|())*\\1", "abc", 0, 0);
 
@@ -524,21 +491,12 @@
   // Compile failed: InvalidEscape("\\Z")
   x2("かきく\\Z", "かきく\n", 0, 9);
 
-  // Compile failed: InvalidEscape("\\G")
-  x2("\\Gぽぴ", "ぽぴ", 0, 6);
-
   // No match found
   x2("(?m:よ.)", "よ\n", 0, 4);
 
   // No match found
   x2("(?m:.め)", "ま\nめ", 3, 7);
 
-  // Compile failed: InvalidEscape("\\G")
-  x2("鬼|\\G車", "け車鬼", 6, 9);
-
-  // Compile failed: InvalidEscape("\\G")
-  x2("鬼|\\G車", "車鬼", 0, 3);
-
   // Compile failed: InvalidEscape("\\Z")
   x2("鬼|車\\Z", "車鬼", 3, 6);
 
